@@ -1,14 +1,30 @@
 package CGI::Formalware;
 
-# -----------------------------------------------------------------
-# Purpose:
-#	Convert an XML file into a suite of CGI forms.
+# Name:
+#	CGI::Formalware.
+#
+# Documentation:
+#	POD-style documentation is at the end. Extract it with pod2html.*.
+#
 #
 # Test environment:
-#	Apache V 1.3.4, 1.3.6, 1.3.9 for Windows.
-# -----------------------------------------------------------------
+#	Apache V 1.3.4, 1.3.6, 1.3.9, 1.3.12 for Windows.
+#
+# Note:
+#	tab = 4 spaces || die.
+#
+# Author:
+#	Ron Savage <ron@savage.net.au>
+#	Home page: http://savage.net.au/index.html
+#
+# Licence:
+#	Australian copyright (c) 1999-2002 Ron Savage.
+#
+#	All Programs of mine are 'OSI Certified Open Source Software';
+#	you can redistribute them and/or modify them under the terms of
+#	The Artistic License, a copy of which is available at:
+#	http://www.opensource.org/licenses/index.html
 
-use integer;
 use strict;
 
 use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);
@@ -23,7 +39,7 @@ require Exporter;
 
 @EXPORT = qw();
 
-$VERSION = '1.08';
+$VERSION = '1.10';
 
 # Preloaded methods go here.
 # -----------------------------------------------------------------
@@ -139,46 +155,46 @@ sub endHandlerPhase2
 
 	if ($element =~ /^form$/i)
 	{
-		# Due to bugs in CGI.pm, the override and default
+		# Due to bugs in CGI.pm (around V 2.56), the override and default
 		# options must appear before the value option.
 		# Also, the override and default options must be used
 		# when there is only 1 radio button. If not, then the
-		# one and only is not selected.
+		# one and only button is not selected.
 		push(@{$$html{$formCountPhase2} }, '<TR>', '<TD>',
-			radio_group({'name' => SCRIPT_HEADING_STRING, 'cols' => 1,
-			'override' => 1,
-			'default' => ${$scriptHeadingMenu{$formCountPhase2} }[0],
-			'value' => \@{$scriptHeadingMenu{$formCountPhase2} } }),
+			radio_group({name => SCRIPT_HEADING_STRING, cols => 1,
+			override => 1,
+			default => ${$scriptHeadingMenu{$formCountPhase2} }[0],
+			value => \@{$scriptHeadingMenu{$formCountPhase2} } }),
 			'</TD>', '</TR>');
 
 		push(@{$$html{$formCountPhase2} }, '<TR>', '<TD>', hr(), p(),
-			submit({'name' => SUBMIT_STRING, 'value' => SUBMIT_STRING}), reset() );
+			submit({name => SUBMIT_STRING, value => SUBMIT_STRING}), reset() );
 
 		# After the first form...
 		if ($formCountPhase2 > 1)
 		{
 			# Add previousForm button.
 			push(@{$$html{$formCountPhase2} },
-				submit({'name' => SUBMIT_STRING, 'value' => PREVIOUS_FORM_STRING}) );
+				submit({name => SUBMIT_STRING, value => PREVIOUS_FORM_STRING}) );
 		}
 
 		# Before the last form...
 		if ($formCountPhase2 < $formCountPhase1)
 		{
 			push(@{$$html{$formCountPhase2} },
-				submit({'name' => SUBMIT_STRING, 'value' => NEXT_FORM_STRING}) );
+				submit({name => SUBMIT_STRING, value => NEXT_FORM_STRING}) );
 		}
 
 		push(@{$$html{$formCountPhase2} }, '</TD>', '</TR>', '</TBODY>',
 			'</TABLE>', '</TD>', '</TR>', '</TABLE>');
 		push(@{$$html{$formCountPhase2} },
-			hidden({'name' => XML_FILE_NAME_STRING, 'value' => $xmlFileName, 'override' => 1}),
-			hidden({'name' => CURRENT_FORM_STRING, 'value' => $formCountPhase2, 'override' => 1}) );
+			hidden({name => XML_FILE_NAME_STRING, value => $xmlFileName, override => 1}),
+			hidden({name => CURRENT_FORM_STRING, value => $formCountPhase2, override => 1}) );
 
 		# End outputting HTML.
 		push(@{$$html{$formCountPhase2} }, end_form(), end_html() );
 
-		&writeFile(undef, $$attribute{'formFileName'}, $$html{$formCountPhase2})
+		$myself -> writeFile($$attribute{'formFileName'}, $$html{$formCountPhase2})
 			if ($myself -> {'form2file'});
 	}
 
@@ -192,17 +208,19 @@ sub endHandlerPhase2
 	# paragraph.
 	if ($element =~ /^paragraph$/i)
 	{
+		my($text) = defined($$attribute{'text'}) ? $$attribute{'text'} : '';
 		push(@{$$html{$formCountPhase2} }, '<TR>', '<TD>',
-			p(), '</TD>', '</TR>');
+			p($text), '</TD>', '</TR>');
 	}
 
 	# radioGroup.
 	if ($element =~ /^radioGroup$/i)
 	{
+		my($columns) = defined($$attribute{'columns'}) ? $$attribute{'columns'} : '1';
 		my(@value) = split(/\|/, $$attribute{'value'});
 		push(@{$$html{$formCountPhase2} }, '<TR>', '<TD>',
 			$$attribute{'prompt'},
-			radio_group({'name' => $$attribute{'name'}, 'value' => \@value}),
+			radio_group({name => $$attribute{'name'}, cols => $columns, value => \@value}),
 			'</TD>', '</TR>');
 	}
 
@@ -213,20 +231,29 @@ sub endHandlerPhase2
 		{
 			push(@{$$html{$formCountPhase2} }, '<TR>', '<TD>',
 				$$attribute{'prompt'},
-				password_field({'name' => $$attribute{'name'},
-				'value' => $$attribute{'value'}, 'size' => $$attribute{'size'} }),
+				password_field({name => $$attribute{'name'},
+				value => $$attribute{'value'}, size => $$attribute{'size'} }),
 				'</TD>', '</TR>');
 		}
 		else
 		{
 			push(@{$$html{$formCountPhase2} }, '<TR>', '<TD>',
 				$$attribute{'prompt'},
-				textfield({'name' => $$attribute{'name'},
-				'value' => $$attribute{'value'}, 'size' => $$attribute{'size'} }),
+				textfield({name => $$attribute{'name'},
+				value => $$attribute{'value'}, size => $$attribute{'size'} }),
 				'</TD>', '</TR>');
 		}
 	}
 
+	# fileField.
+	if ($element =~ /^fileField$/i)
+	{
+		push(@{$$html{$formCountPhase2} }, '<TR>', '<TD>',
+			$$attribute{'prompt'},
+			filefield({name => $$attribute{'name'}, size => $$attribute{'size'} }),
+			'</TD>', '</TR>');
+	}
+
 	# scripts.
 	if ($element =~ /^scripts$/i)
 	{
@@ -239,7 +266,7 @@ sub endHandlerPhase2
 	{
 		my($script) = [];
 
-		for (sort(keys(%$attribute) ) )
+		for (sort keys %$attribute)
 		{
 			push(@$script, $$attribute{$_}) if (/^line\d{1,2}$/i);
 		}
@@ -255,7 +282,7 @@ sub endHandlerPhase2
 	}
 
 	# Gather field names for substitution into scripts.
-	if ( ($element =~ /^(radioGroup|textField)$/i) )
+	if ( ($element =~ /^(fileField|radioGroup|textField)$/i) )
 	{
 		${$fieldName{$formCountPhase2} }{$$attribute{'name'} } = $1;
 	}
@@ -305,14 +332,19 @@ sub getXMLFileName
 	# Use "SUBMIT_STRING ", not just SUBMIT_STRING, so the if test in the main line,
 	# if ($submission eq SUBMIT_STRING), returns false.
 
-	print header(), start_html('title' => $heading),
+	print header(), start_html(title => $heading),
 		start_form(), h1($heading), hr(),
-		'Please enter the name of the XML file which will be used to generate the CGI forms: ',
-		hidden({'name' => CURRENT_FORM_STRING, 'value' => 1, 'override' => 1}),
-		p(), textfield({'name' => XML_FILE_NAME_STRING, 'value' => '', 'size' => 40, 'override' => 1}),
-		hr(), submit({'name' => SUBMIT_STRING, 'value' => (SUBMIT_STRING . ' ')}),
+		'Please enter the name of the XML file which will be used to generate the CGI forms. ',
+		p(),
+		'This can be any DOS file path, eg D:\Temp\menu.xml. Also, .\menu.xml uses cgi-bin\: ',
+		hidden({name => CURRENT_FORM_STRING, value => 1, override => 1}),
+		p(), textfield({name => XML_FILE_NAME_STRING, value => '', size => 50, override => 1}),
+		hr(), submit({name => SUBMIT_STRING, value => (SUBMIT_STRING . ' ')}),
 		end_form(), end_html();
 
+#		<FORM ENCTYPE="multipart/form-data" ACTION="/cgi-bin/upload.cgi" METHOD="POST">
+#		<INPUT TYPE="FILE" NAME="file-to-upload-01" SIZE="35">
+
 }	# End of getXMLFileName.
 
 #-------------------------------------------------------------------
@@ -335,16 +367,16 @@ sub new
 
 sub phase1
 {
-	my($self) = @_;
+	my($self, $xmlFileName) = @_;
 
 	# Declare the parser.
 	my($parser)	= new XML::DOM::Parser;
 
 	$parser -> setHandlers
 	(
-		'Start'	=> \&startHandlerPhase1,
-		'End'	=> \&endHandlerPhase1,
-		'Char'	=> \&charHandlerPhase1,
+		Start	=> \&startHandlerPhase1,
+		End		=> \&endHandlerPhase1,
+		Char	=> \&charHandlerPhase1,
 	);
 
 	# Parse the document and call the handlers.
@@ -356,16 +388,16 @@ sub phase1
 
 sub phase2
 {
-	my($self) = @_;
+	my($self, $xmlFileName) = @_;
 
 	# Declare the parser.
 	my($parser)	= new XML::DOM::Parser;
 
 	$parser -> setHandlers
 	(
-		'Start'	=> \&startHandlerPhase2,
-		'End'	=> \&endHandlerPhase2,
-		'Char'	=> \&charHandlerPhase2,
+		Start	=> \&startHandlerPhase2,
+		End		=> \&endHandlerPhase2,
+		Char	=> \&charHandlerPhase2,
 	);
 
 	# Parse the document and call the handlers.
@@ -401,10 +433,10 @@ sub process
 		croak("Can't find XML file: $xmlFileName\n") if (! -e $xmlFileName);
 
 		# Parse once to count things.
-		&phase1($self);
+		$self -> phase1($xmlFileName);
 
 		# Parse again to do things.
-		&phase2($self);
+		$self -> phase2($xmlFileName);
 
 		my($submission) = param(SUBMIT_STRING) || '';
 
@@ -414,15 +446,24 @@ sub process
 			my($scriptHeading)	= param(SCRIPT_HEADING_STRING);
 			my($formCount)		= param(CURRENT_FORM_STRING);
 			my($scriptType)		= ${$scriptType{$formCount} }{$scriptHeading};
-			&runLocalScript($self, $scriptHeading, $formCount)	if ($scriptType =~ /^local$/i);
-			&runRemoteScript($self, $scriptHeading, $formCount)	if ($scriptType =~ /^remote$/i);
+			$self -> runLocalScript($scriptHeading, $formCount)		if ($scriptType =~ /^local$/i);
+			$self -> runRemoteScript($scriptHeading, $formCount)	if ($scriptType =~ /^remote$/i);
 		}
 		else
 		{
 			# Send a stored form to the browser.
+			# This is the non-TOC code.
 			my($next)	= param(CURRENT_FORM_STRING);
-			$next		-= 1	if (param(SUBMIT_STRING) eq PREVIOUS_FORM_STRING);
-			$next		+= 1	if (param(SUBMIT_STRING) eq NEXT_FORM_STRING);
+			$next		-= 1	if ($submission eq PREVIOUS_FORM_STRING);
+			$next		+= 1	if ($submission eq NEXT_FORM_STRING);
+
+			# This is the TOC code.
+			my($page);
+
+			for ($page = 1; $page <= ($#tableOfContents + 1); $page++)
+			{
+				$next = $page if (param($page) );
+			}
 
 			for (@{$$html{$next} })
 			{
@@ -432,7 +473,7 @@ sub process
 	}
 	else
 	{
-		&getXMLFileName($self);
+		$self -> getXMLFileName();
 	}
 
 	close(DEBUG) if ($self -> {'debug'});
@@ -473,10 +514,10 @@ sub runLocalScript
 
 	my($script)			= ${$script{$formCount} }{$heading};
 	my($fieldName)		= $fieldName{$formCount};
-	@$script			= &expandMacros($self, $script, $fieldName);
+	@$script			= $self -> expandMacros($script, $fieldName);
 	my($scriptFileName)	= 'script.bat';
 
-	&writeFile($self, $scriptFileName, $script);
+	$self -> writeFile($scriptFileName, $script);
 
 	# Handle FTP here.
 	if ($$script[$#{$script}] =~ /^ftp\s+(-n\s+-v|-v\s+-n)$/i)
@@ -491,9 +532,9 @@ sub runLocalScript
 		close(OUT);
 
 		# Patch batch file to run FTP.
-		my($cmd)		= &readFile($self, $scriptFileName);
+		my($cmd)		= $self -> readFile($scriptFileName, 1);
 		$$cmd[$#{$cmd}]	= "type $scriptFileName.txt | $$script[$#{$script}]\n";
-		&writeFile($self, $scriptFileName, $cmd);
+		$self -> writeFile($scriptFileName, $cmd);
 	}
 
 	my($startTime)	= time();
@@ -507,11 +548,11 @@ sub runLocalScript
 	for (@log)
 	{
 		# Convert anything which looks like an HTML tag to something else.
-		# In particular, DOS directory listings containing 'XYZ <DIR>'.
-		s/<(.+?)>/{$1}/g;
+		# In particular, XML, & DOS dir listings containing 'cgi-bin <DIR>'.
+		s/<(.+?)>/&lt;$1&gt;/g;
 		print $_;
 	}
-	
+
 	1 while ($time =~ s/(\d)(\d\d\d)(?!\d)/$1,$2/);
 
 	if ($self -> {'timeScripts'})
@@ -534,10 +575,10 @@ sub runRemoteScript
 
 	my($script)			= ${$script{$formCount} }{$heading};
 	my($fieldName)		= $fieldName{$formCount};
-	@$script			= &expandMacros($self, $script, $fieldName);
+	@$script			= $self -> expandMacros($script, $fieldName);
 	my($scriptLogName)	= "script.log";
 	my($prompt)			= '/\[\d+\] /';
-	my($firstCommand)	= '';
+	my($actualCommand)	= '';
 	my($startTime)		= time();
 
 	unlink($scriptLogName);
@@ -555,12 +596,12 @@ sub runRemoteScript
 		$session -> open(param('host') );
 		$session -> login(param('username'), param('password') );
 
-# Warning: Use spaces and not tabs to separated fields within these strings.
-#	Net::Telnet strips the tabs and does not replace them with a space.
+		# Warning: Use spaces and not tabs to separated fields within these strings.
+		#	Net::Telnet V 3.01 strips the tabs and does not replace them with a space.
 
 		for (@$script)
 		{
-			$firstCommand = $_ if (! $firstCommand);
+			$actualCommand = $_ if (! $actualCommand);
 			$session -> cmd($_);
 		}
 
@@ -568,23 +609,28 @@ sub runRemoteScript
 	}
 
 	my($time)	= time() - $startTime;
-	my($log)	= &readFile($self, $scriptLogName);
+	my($log)	= $self -> readFile($scriptLogName, 1);
 
 	unlink($scriptLogName);
 
 	print header(), start_html($heading), start_form(), h1($heading), hr(),
-		hidden({'name' => SUBMIT_STRING, 'value' => SUBMIT_STRING, 'override' => 1}),
+		hidden({name => SUBMIT_STRING, value => SUBMIT_STRING, override => 1}),
 		'<PRE>';
 
-	my($first)		= 0;
-	$firstCommand	= quotemeta($firstCommand);
+	# Print nothing until we find the command we sent.
+	my($foundCommand)	= 0;
+	$actualCommand		= quotemeta($actualCommand);
 
 	for (@$log)
 	{
-		$first = 1 if (/$firstCommand\s*$/);
-		print "$_\n" if ($first);
+		# Convert anything which looks like an HTML tag to something else.
+		# In particular, XML, & DOS dir listings containing 'cgi-bin <DIR>'.
+		s/<(.+?)>/&lt;$1&gt;/g;
+
+		$foundCommand = 1 if (/$actualCommand\s*$/);
+		print "$_\n" if ($actualCommand);
 	}
-	
+
 	1 while ($time =~ s/(\d)(\d\d\d)(?!\d)/$1,$2/);
 
 	if ($self -> {'timeScripts'})
@@ -647,9 +693,17 @@ sub startHandlerPhase1
 				! defined($attribute{'size'}) );
 	}
 
-	if ($element =~ /^(radioGroup|textField)$/i)
+	if ($element =~ /^fileField$/i)
 	{
-		croak("Each radioGroup & textField entity must have a unique 'name' attribute (per form)\n")
+		croak("Each fileField entity must have 'name', 'prompt' and 'size' attributes\n")
+			if (! defined($attribute{'name'}) ||
+				! defined($attribute{'prompt'}) ||
+				! defined($attribute{'size'}) );
+	}
+
+	if ($element =~ /^(fileField|radioGroup|textField)$/i)
+	{
+		croak("Each fieldField, radioGroup & textField entity must have a unique 'name' attribute (per form)\n")
 			if (defined($fieldNameSeen{$formCountPhase1}{$attribute{'name'} }) );
 
 		$fieldNameSeen{$formCountPhase1}{$attribute{'name'} } = $formCountPhase1;
@@ -702,7 +756,7 @@ sub startHandlerPhase2
 	{
 		$formCountPhase2++;
 
-		my($author)								= (defined($attribute{'author'}) ? $attribute{'author'} : 'Ron Savage');
+		my($author)								= (defined($attribute{'author'}) ? $attribute{'author'} : 'ron@savage.net.au');
 		my($heading)							= $attribute{'heading'};
 
 		$fieldName{$formCountPhase2}			= {};
@@ -714,13 +768,13 @@ sub startHandlerPhase2
 		$scriptType{$formCountPhase2}			= {};
 
 		my($startHtml)							=
-			start_html({'title' => $heading, 'author' => $author} );
+			start_html({title => $heading, author => $author} );
 		$startHtml								=
-			start_html({'title' => $heading, 'author' => $author,
-			'style' => {'src' => $attribute{'css'} } } ) if (defined($attribute{'css'}) );
+			start_html({title => $heading, author => $author,
+			style => {src => $attribute{'css'} } } ) if (defined($attribute{'css'}) );
 
 		# Output the Table of Contents.
-		push(@{$$html{$formCountPhase2} }, header(), $startHtml, start_form(),
+		push(@{$$html{$formCountPhase2} }, header(), $startHtml, start_multipart_form(),
 			'<TABLE WIDTH = "100%">');
 
 		if ($tableOfContents{'tocVisible'} =~ /True/i)
@@ -731,7 +785,7 @@ sub startHandlerPhase2
 			if (defined($attribute{'css'}) )
 			{
 				push(@{$$html{$formCountPhase2} },
-					p({'class' => 'TOC'}, $tableOfContents{'tocEntry'}) );
+					p({class => 'TOC'}, $tableOfContents{'tocEntry'}) );
 			}
 			else
 			{
@@ -740,17 +794,25 @@ sub startHandlerPhase2
 
 			push(@{$$html{$formCountPhase2} }, '</TD>', '</TR>', '</THEAD>', '<TBODY>');
 
+			my($page) = 0;
+
 			for (@tableOfContents)
 			{
+				$page++;
+
 				if (defined($attribute{'css'}) )
 				{
 					push(@{$$html{$formCountPhase2} }, '<TR>', '<TD>',
-						p({'class' => 'TOC'}, $_), '</TD>', '</TR>');
+#						p({class => 'TOC'}, $_),
+						submit({class => 'TOC', name => $page, value => $_}),
+						'</TD>', '</TR>');
 				}
 				else
 				{
 					push(@{$$html{$formCountPhase2} }, '<TR>', '<TD>',
-						$_, '</TD>', '</TR>');
+#						$_,
+						submit({name => $page, value => $_}),
+						'</TD>', '</TR>');
 				}
 			}
 
@@ -803,12 +865,11 @@ In your browser, type: localhost/cgi-bin/x.pl
 where x.pl contains nothing more than:
 
 	#!perl -w
-	use integer;
 	use strict;
 	use lib 'C:/Perl';
 	use lib 'C:/Perl/Scripts/General';	# Ie $PERL5LIB.
 	use CGI::Formalware;
-	my($form) = CGI::Formalware -> new({'form2file' => 1, 'debug' => 1});
+	my($form) = CGI::Formalware -> new({form2file => 1, debug => 1});
 	$form -> process();
 	exit(0);
 
@@ -865,18 +926,18 @@ Keys and values recognized are:
 
 =item *
 
-'debug' => '1' means turn on debugging. At the moment this opens and closes the
+debug => 1 means turn on debugging. At the moment this opens and closes the
 file CGI-Formalware.log, but does not write anything to it
 
 =item *
 
-'form2file' => '1' means output each form to a file, using the name given
+form2file => 1 means output each form to a file, using the name given
 by the form's formFileName attribute. The forms are written to cgi-bin/.
 If the form has no such attribute, this option is ignored. See example below
 
 =item *
 
-'timeScripts' => '1' means report elapsed time at the end of each script's output
+timeScripts => 1 means report elapsed time at the end of each script's output
 
 =back
 
@@ -902,9 +963,62 @@ Each form is more-or-less assumed to contain a list of scripts
 
 =item *
 
-Entities in the XML correspond to a few functions available in Lincoln Stein's
-CGI.pm. Eg: textField, radioGroup, horizontalRule. Over time, more functions will
-be added
+Tokens in the XML correspond to a few functions available in Lincoln Stein's
+CGI.pm. Available tokens are:
+
+=over 4
+
+=item *
+
+fileField
+
+	<fileField
+		name		= 'fileName'
+		prompt		= 'Filename: '
+		size		= '60'
+		override	= '0'
+	/>
+
+=item *
+
+horizontalRule
+
+	<horizontalRule />
+
+=item *
+
+paragraph
+
+	<paragraph />
+
+	<paragraph text = 'Output a comment' />
+
+=item *
+
+radioGroup
+
+	<radioGroup
+		name		= 'serverName'
+		prompt		= 'Server name: '
+		value		= 'Example|Simple|Test'
+		columns		= '1'	# Optional. Defaults to '1'. Use a string, not a digit
+	/>
+
+=item *
+
+textField
+
+	<textField
+		name		= 'username'
+		prompt		= 'Username: '
+		value		= ''
+		size		= '15'
+		override	= '0'
+	/>
+
+=back
+
+Over time, more functions will be added.
 
 =item *
 
@@ -1019,11 +1133,11 @@ Each form entity must have a unique 'heading' attribute
 
 Each form entity may have a unique 'formFileName' attribute. If present, then
 this file name is used to output the form to a file if the constructor option
-new({'form2file' => 1}) is used
+new({form2file => 1}) is used
 
 =item *
 
-Each textField entity must have 'name', 'prompt', and 'value' attributes
+Each fileField entity must have 'name', 'prompt', 'value' and 'size' attributes
 
 =item *
 
@@ -1065,7 +1179,7 @@ Herewith a sample:
 		css				= '/CGI-Formalware.css'
 		formFileName	= '1.html'
 	>
-		<horizontalRule/>
+		<horizontalRule />
 
 		<radioGroup
 			name	= 'host'
@@ -1073,7 +1187,7 @@ Herewith a sample:
 			value	= 'bigBox|littleBox'
 		/>
 
-		<paragraph/>
+		<paragraph />
 
 		<textField
 			name		= 'username'
@@ -1091,7 +1205,7 @@ Herewith a sample:
 			override	= '0'
 		/>
 
-		<horizontalRule/>
+		<horizontalRule />
 
 		<scripts
 			heading			= 'Unix Scripts'
@@ -1116,7 +1230,7 @@ Herewith a sample:
 		tocEntry	= 'DOS menu'
 	>
 
-		<horizontalRule/>
+		<horizontalRule />
 
 		<radioGroup
 			name	= 'host'
@@ -1124,7 +1238,7 @@ Herewith a sample:
 			value	= 'bigBox|littleBox'
 		/>
 
-		<paragraph/>
+		<paragraph text = 'Enter a username and a password.' />
 
 		<textField
 			name		= 'username'
@@ -1142,17 +1256,16 @@ Herewith a sample:
 			override	= '0'
 		/>
 
-		<horizontalRule/>
+		<horizontalRule />
 
-		<textField
+		<fileField
 			name		= 'fileName'
 			prompt		= 'Filename: '
-			value		= ''
 			size		= '60'
 			override	= '0'
 		/>
 
-		<horizontalRule/>
+		<horizontalRule />
 
 		<scripts
 			heading	= 'PC Scripts'
@@ -1201,23 +1314,20 @@ XML::DOM
 
 =back
 
-=head1 TO DO
-
-DOS directory text '... <DIR>' is changed to '... {DIR}', so that it
-does not look like HTML. This needs to be handled more cleanly.
-
 =head1 AUTHOR
 
 C<CGI::Formalware> was written by Ron Savage I<E<lt>ron@savage.net.auE<gt>>
 in 1999.
 
-Copyright (c) 1999 Ron Savage.
-
 Available from http://savage.net.au/Perl.html.
 
 =head1 LICENCE
 
-This program is free software; you can redistribute it and/or modify it under
-the same terms as Perl itself.
+Australian copyright (c) 1999 Ron Savage.
+
+	All Programs of mine are 'OSI Certified Open Source Software';
+	you can redistribute them and/or modify them under the terms of
+	The Artistic License, a copy of which is available at:
+	http://www.opensource.org/licenses/index.html
 
 =cut
